/** =======================================================================
 *  Vincent Hardy
 *  License terms: see svg-wow.org
 *  CC0 http://creativecommons.org/publicdomain/zero/1.0/
 *  ======================================================================= */

function onReady(Y) {
    // =========================================================================
    // Graffiti renderers.
    //
    // Renderers are called as the graffiti is drawn. They can render when a
    // point is updated (the update method), skipped (the skipPoint method)
    // or after all the graffiti has been drawn (the fadeRendering method).
    // =========================================================================
    var StrokeDashAnimation = {
        initialize : function (state) {
            var paths = state.paths, n = paths.length;
            for (var si = 0; si < n; si++) {
                paths[si].setAttribute("stroke-dasharray",
                                       paths[si]._pathLength + ',' +
                                       paths[si]._pathLength * 2);
                paths[si].setAttribute('stroke-dashoffset',
                                       paths[si]._pathLength * 1.5);
                this._paths = paths;
            }
        },

        update : function (curStroke, curPoint, progress) {
            this._paths[curStroke].setAttribute("stroke-dashoffset",
                this._paths[curStroke]._pathLength * (1 - progress));
        },

        skipPoint : function (point, curStroke) {
            // Do nothing. See update method.
        },

        fadeRendering : function () {
            return false; // Nothing to render after the end of the animation.
        }
    };

    var ParticlesAnimation = {
        initialize : function (state) {
            this._state = state;
            this._particles = [];
        },

        update : function (curStroke, curPoint, progress) {
            this.fadeParticles(this._particles);
        },


        skipPoint : function (point, curStroke) {
            this.addPointParticles(point,
                              this._state.matrix,
                              this._state.origin,
                              this._particles);
        },

        fadeRendering: function () {
            var particles = this._particles, hasMore = true;
            if (particles.length === 0) {
                hasMore = false;
            } else {
                this.fadeParticles(particles);
            }
            return hasMore;
        },

        /**
         * Draws the particles at their current position.
         */
       fadeParticles: function (particles) {
            var nParticles = particles.length,
            particle,
            decel;

            for (var pi = nParticles - 1; pi >= 0; pi--) {
                particle = particles[pi];
                decel = (particle.maxIter - particle.i) / particle.maxIter;
                particle.i++;
                particle.x += decel * particle.dx;
                particle.y += decel * particle.dy;

                ctx.globalAlpha = (particle.maxIter - particle.i) / particle.maxIter;
                ctx.fillRect(particle.x, particle.y, particle.size, particle.size);
                if (particle.i >= particle.maxIter) {
                    particles.splice(pi, 1);
                }
            }
        },

        /**
         * Adds particles to the particles array for the given point.
         *
         * @param point the point for which particles should be added.
         * @param matrix the transform between the graffiti's coordinate system
         *        and the SVG coordinate system.
         * @param origin the graffiti's origin in the SVG coordinate system.
         * @param particles the array of particles to append to.
         */
        addPointParticles: function (point, matrix, origin, particles) {
            var speed = {
                x: point.speedOut.sx,
                y: point.speedOut.sy
            },
            s = {},
            pt = {};
            matrix.transformPoint(point, pt);

            // Transform the speed vector in the target coordinate system
            // We transform the vector ({0,0}, {speed.x, speed.y}) and
            // compute its x and y axis values.
            matrix.transformPoint(speed, s);

            speed = {
                x: s.x - origin.x,
                y: s.y - origin.y
            };

            // Convert the speed to polar coordinates
            var theta = Math.atan2(speed.y, speed.x),
            delta = Math.PI / 10,
            alpha,
            r = Math.sqrt(speed.x * speed.x +
                speed.y * speed.y),
            rad;

            var nP = Math.round(MIN_PARTICLES + DELTA_PARTICLES * Math.random());
            for (var pi = 0; pi < nP; pi++) {
                alpha = theta + delta * (1 - 2 * Math.random());
                rad = r * (0.8 + 0.8 * Math.random());

                particles.push({
                    x: pt.x,
                    y: pt.y,
                    dx: rad * Math.cos(alpha) * animationRateSec,
                    dy: rad * Math.sin(alpha) * animationRateSec,
                    i: 1,
                    maxIter: PARTICLE_ITERATIONS,
                    size: PARTICLE_SIZE
                });
            }
        }
    };

    /**
     * PatchRenderingAnimation is an ugly hack to make up for a bug found in
     * Safari 5.0, Firefox 3.6 and Chrome with line joins which get 'cut-off'
     * and create a nasty rendering result.
     */
    var PatchRenderingAnimation = {
        _state: null,

        initialize: function (state) {
            this._state = state;
        },

        update : function (curStroke, curPoint, progress) {
        },

        skipPoint : function (point, curStroke) {
            var pt = this._state.matrix.transformPoint(point),
                outerPatchId = "#stroke_" + curStroke + "_outerPatch",
                innerPatchId = "#stroke_" + curStroke + "_innerPatch";
            Y.one(outerPatchId).loadContent({
                tag: 'circle',
                cx: pt.x,
                cy: pt.y,
                r: OUTER_WIDTH / 2,
                fill: 'black',
                stroke: 'none'
            });
            Y.one(innerPatchId).loadContent({
                tag: 'circle',
                cx: pt.x,
                cy: pt.y,
                r: INNER_WIDTH / 2,
                fill: 'white',
                stroke: 'none'
            });
        },

        fadeRendering : function () {
            return false;
        }
    };

    var MultiplexAnimations = {
        animations: [StrokeDashAnimation, ParticlesAnimation, PatchRenderingAnimation],

        initialize: function (state) {
            
            for (var i = 0; i < this.animations.length; i++) {
                this.animations[i].initialize(state);
            }
        },

        update: function (curStroke, curPoint, progress) {
            for (var i = 0; i < this.animations.length; i++) {
                this.animations[i].update(curStroke, curPoint, progress);
            }
        },

        fadeRendering : function () {
            var result = false;
            for (var i = 0; i < this.animations.length; i++) {
                result |= this.animations[i].fadeRendering();
            }
            return result;
        },

        skipPoint : function (point, curStroke) {
            for (var i = 0; i < this.animations.length; i++) {
                this.animations[i].skipPoint(point, curStroke);
            }
        }
    };

    // =========================================================================
    // Configuration parameters
    // =========================================================================

    /**
     * The minimum and delta number of particles per graffiti point
     */
    var MIN_PARTICLES = 20,
    DELTA_PARTICLES = 20,

    /**
     * The number of iterations before a particle stops
     */
    PARTICLE_ITERATIONS = 15,

    /**
     * Margin around the graffiti
     */
    MARGIN = 100,

    /**
     * Size of the particle rectangles
     */
    PARTICLE_SIZE = 1,

    /**
     * Color used to draw particles
     */
    PARTICLE_COLOR = 'white',

    /**
     * Color used to record graffitis
     */
    RECORD_COLOR = 'crimson',

    /**
     * Size used for record markers
     */
    RECORD_MARKER_SIZE = 3;

    /**
     * Default renderer
     */
    DEFAULT_RENDERER = MultiplexAnimations,

    /**
     * Stroke widths, inner and outer
     */
    INNER_WIDTH = 14,
    OUTER_WIDTH = 18,

    // =========================================================================
    // Misc scope variables, used accross graffitis
    // =========================================================================

    /**
     * The group holding the SVG graffiti
     */
    g = Y.one("#graffiti"),

    /**
     * Particles canvas.
     */
    canvas = Y.one('#graffitis-canvas'),
    ctx = Y.Node.getDOMNode(canvas).getContext('2d'),

    /**
     * The target size for the graffiti.
     */
    targetWidth = g.get('nearestViewportElement').get('width') - MARGIN,
    targetHeight = g.get('nearestViewportElement').get('height') - MARGIN,

    /**
     * The delay between animation frames.
     */
    animationRate = 20,
    animationRateSec = 20 / 1000,

    /**
     * Recorder: used to record a graffiti
     */
    recorder = null;

    /**
     * The current graffiti renderer
     */
    renderer = DEFAULT_RENDERER;


    // =========================================================================
    // Graffiti state
    // =========================================================================
    var graffitiState = {
        /**
          * The current GML result
          */
        gmlResult: null
    };
    
    // =========================================================================
    // Animation state
    // =========================================================================
    var animationState = {
        /**
         * Currently painted stroke
         */
        curStroke: 0,

        /**
         * End of the current segment on the current stroke.
         */
        curPoint: 0,

        /**
         * Current setInterval id
         */
        intervalId: -1
    };
    
    /**
     * There are different ways to specify the rotation angle in GML. This
     * takes care of the differences between versions.
     */
    function extractRotation (gmlResult) {
        var rotation = 0, env = gmlResult.gmlData.gml.tag.environment, up;
        if (env !== undefined) {
            if (env.up !== undefined) {
                up = env.up;
            }

            if (up !== undefined) {
                rotation = -90 + Math.atan2(up.y, up.x) * 180 / Math.PI;
            }
        }
        return rotation;
    }

    /**
     * Computes the transform matrix for the given graffiti.
     * @param glmResult
     * @return the transform from graffiti space to the SVG space.
     */
    function computeGraffitiTransform (gmlResult) {
        var bbox = gmlResult.gmlMetrics.bounds,
        sx = 1, sy = 1, s = 1, rotation = 0,
        env = gmlResult.gmlData.gml.tag.environment;

        // Compute the scale factor for the graffiti: use the targetWidth and height
        // and the bounding box of the computed graffiti bounds.
        if (bbox.width > 0) {
            sx = targetWidth / bbox.width;
        }

        if (bbox.height > 0) {
            sy = targetHeight / bbox.height;
        }

        s = sx;

        if (sy < sx) {
            s = sy;
        }

        // Compute the aspect ratio based on the environment's width and height
        var ar = 1;

        if (env.screen_bounds !== undefined) {
            ar = env.screen_bounds.x / env.screen_bounds.y;
        }

        // Compute the rotation from the gmlData.
        rotation = extractRotation(gmlResult);

        var matrix = new Y.DOM.Matrix();
        matrix.translate(((MARGIN + targetWidth) / 2),
            ((MARGIN + targetHeight) / 2));
        matrix.rotate(rotation);
        matrix.scaleNonUniform(s, s * ar);
        matrix.translate((-bbox.x - bbox.width / 2),
            (-bbox.y - bbox.height / 2));

        return matrix;
    }

    /**
     * Used to lower the decimal precision on path data.
     */
    function round (v) {
        return Math.round(v * 10000) / 10000;
    }

    /**
     * Creates the graffiti shapes as SVG path elements.
     *
     * @param gmlResult
     * @return an array of path elements.
     */
    function createGraffitiPaths (gmlResult) {
        var strokes = gmlResult.gmlMetrics.strokes,
        nStrokes = strokes.length,
        points,
        segments,
        nSegments,
        matrix,
        seg,
        d,
        paths = [],
        strokeCounter = 0,
        pathLength = 0;

        matrix = computeGraffitiTransform(gmlResult);

        
        for (var si = 0; si < nStrokes; si++) {
            points = strokes[si].pt;
            segments = smooth(points, 0.75);
            nSegments = segments.length;
            d = "";

            segments[0][0] = matrix.transformPoint(segments[0][0]);
            if (nSegments > 0) {
                d = "M" + round(segments[0][0].x) + "," +
                          round(segments[0][0].y);
            }

            for (var i = 0; i < nSegments; i++) {
                seg = segments[i];

                seg[1] = matrix.transformPoint(seg[1]);
                seg[2] = matrix.transformPoint(seg[2]);
                seg[3] = matrix.transformPoint(seg[3]);

                d = d.concat("C", round(seg[1].x), ",", round(seg[1].y), " ",
                    round(seg[2].x), ",", round(seg[2].y), " ",
                    round(seg[3].x), ",", round(seg[3].y));
            }

            g.loadContent({
                tag: 'g',
                id: "stroke_" + strokeCounter + "_outerPatch"
            });
            g.loadContent({
                tag: "path",
                id: "stroke_" + strokeCounter,
                d: d
            });

            paths.push(g.get('lastChild'));
            paths[si]._pathLength = Y.Node.getDOMNode(paths[si]).getTotalLength();
            pathLength += paths[si]._pathLength;

            g.loadContent({
                tag: 'g',
                id: "stroke_" + strokeCounter + "_innerPatch"
            });
            g.loadContent({
                tag: "use",
                "xlink:href": "#stroke_" + strokeCounter,
                'class': 'inner-stroke'
            });


            strokeCounter++;
        }

        return paths;
    }

    /**
     * Exports the current graffiti to SVG
     */
    function exportToSVG () {
        if (graffitiState.paths !== undefined) {
            // Create a simple SVG file with the path data.
            var svg = Y.one('#graffitiViewer'),
                bounds = svg._node.getBoundingClientRect(),
                width = Math.round(bounds.right - bounds.left),
                height = Math.round(bounds.bottom - bounds.top),
                paths = graffitiState.paths, n = paths.length,
                Animate = sw.animation.Animate, Easing = Y.Easing;

            var svgContent = [
                    "<svg width='", width, "' ",
                    "height='", height + "' ",
                    "xmlns='", Y.svgNS + "' ",
                    "stroke='black' fill='none' stroke-width='" +
                                                            OUTER_WIDTH + "' ",
                    "stroke-linecap='round' stroke-linejoin='round' ",
                    "viewBox='0 0 ", width, ' ', height, "'>\n"];

            for (var i = 0; i < n; i++) {
                svgContent.push("<path d='");
                svgContent.push(paths[i]._node.getAttribute('d'));
                svgContent.push("' />\n");
            }

            svgContent.push("</svg>");
            Y.one('#svg-text').set('textContent', svgContent.join(''));

            // Animate showing the svg content in a dialog.
            var headerBounds = Y.one('#header')._node.getBoundingClientRect(),
                svgCapture = Y.one('#svg-capture').setStyle('display', 'block'),
                captureBounds = svgCapture._node.getBoundingClientRect(),
                captureWidth = captureBounds.right - captureBounds.left,
                headerWidth = headerBounds.right - headerBounds.left;

            svgCapture.setStyle('left', headerBounds.left +
                                        (headerWidth - captureWidth) / 2);

            var showSVGCapture = new Animate({
                node: svgCapture,
                from: {
                    top: -captureBounds.height
                },
                to: {
                    top: "50px"
                },
                duration: 2,
                easing: Easing.elasticOut
            });

            showSVGCapture.run();
        }
    }

    /**
     * Show the current graffiti's GML
     */
    function showGML () {
        if (graffitiState.paths !== undefined) {
            // Create a simple SVG file with the path data.
            var Animate = sw.animation.Animate, Easing = Y.Easing;

            if (graffitiState.gmlResult !== undefined) {
                var gmlContent = JSON.stringify(graffitiState.gmlResult.gmlData);

                Y.one('#gml-text').set('textContent', gmlContent);

                // Animate showing the svg content in a dialog.
                var headerBounds = Y.one('#header')._node.getBoundingClientRect(),
                    gmlCapture = Y.one('#gml-capture').setStyle('display', 'block'),
                    captureBounds = gmlCapture._node.getBoundingClientRect(),
                    captureWidth = captureBounds.right - captureBounds.left,
                    headerWidth = headerBounds.right - headerBounds.left;

                gmlCapture.setStyle('left', headerBounds.left +
                                            (headerWidth - captureWidth) / 2);

                var showSVGCapture = new Animate({
                    node: gmlCapture,
                    from: {
                        top: -captureBounds.height
                    },
                    to: {
                        top: "50px"
                    },
                    duration: 2,
                    easing: Easing.elasticOut
                });

                showSVGCapture.run();
            } else {
                alert("There is no graffiti displayed yet. Display one before" +
                      " showing its GML source");
            }
        }
    }

    /**
     * Shows the current graffiti in the overlay panel, demonstrating a
     * sophisticated filter effect.
     */
    function showOverlay () {
        var headerBounds = Y.one('#header')._node.getBoundingClientRect(),
        brickOverlay = Y.one('#brick-overlay').setStyle('display', 'block'),
        overlayBounds = brickOverlay._node.getBoundingClientRect(),
        overlayWidth = overlayBounds.right - overlayBounds.left,
        headerWidth = headerBounds.right - headerBounds.left;

        brickOverlay.setStyle('left', headerBounds.left +
            (headerWidth - overlayWidth) / 2);

        var showSVGCapture = new sw.animation.Animate({
            node: brickOverlay,
            from: {
                top: -overlayBounds.height
            },
            to: {
                top: "50px"
            },
            duration: 2,
            easing: Y.Easing.elasticOut
        });

        showSVGCapture.run();

        // Show the current strokes in the overlay content.
        var overlayContent = Y.one('#stroke-paths'),
            paths = graffitiState.paths, n = paths.length;

        overlayContent.removeAllChildren();
        for (var i = 0; i < n; i++) {
            overlayContent.loadContent({
                tag: 'path',
                d: paths[i].getAttribute('d')
            });
        }
    }

    /**
     * The main animation method.
     *
     */
    function animateStroke () {
        var curTime = (new Date()).getTime(),
        elapsed = (curTime - animationState.startTime) / 1000, // ms to s
        progress,
        point, prevPoint,
        segmentProgress,
        allStrokesConsumed = false,
        curStroke = animationState.curStroke,
        curPoint = animationState.curPoint,
        strokesInfo = animationState.strokesInfo,
        gmlResult = graffitiState.gmlResult,
        points,
        paths = graffitiState.paths;

        // Find out which stroke we are animating.
        while (strokesInfo[curStroke].tagEndTimeOffset < elapsed) {
            // We are past the duration of the current stroke.

            // Skip all the remaining points on the stroke.
            points = strokesInfo[curStroke].points;
            for (var i = curPoint; i < points.length - 1; i++) {
                renderer.skipPoint(points[i], curStroke);
            }

            // Make sure the current stroke's dashoffset is properly set.
            paths[curStroke].removeAttribute('stroke-dashoffset');
            paths[curStroke].removeAttribute('stroke-dasharray');
            curStroke++;
            curPoint = 0;

            if (curStroke === gmlResult.gmlMetrics.strokes.length) {
                // We reached the end of the last stroke. Stop the animation
                // now.
                clearInterval(animationState.intervalId);

                // Let the renderers finish their rendering.
                animationState.intervalId = setInterval(fadeRendering, animationRate);
                startTime = (new Date()).getTime()
                allStrokesConsumed = true;
                break;
            }
        }

        if (allStrokesConsumed === false) {
            points = strokesInfo[curStroke].points;

            // Find out which stroke point we are on.
            while (points[curPoint].tagTimeOffset < elapsed) {
                renderer.skipPoint(points[curPoint], curStroke);
                curPoint++;
            // Can we have reached the end of the current stroke?
            // No because we checked that we did not go beyond the end of
            // the stroke right before.
            }

            // We are in the segment ending on 'point'. Compute the
            // penetration in the current segment and then the current
            // position.
            if (curPoint > 0) {
                point = points[curPoint];
                prevPoint = points[curPoint - 1];
                segmentProgress = (elapsed - prevPoint.tagTimeOffset) /
                (point.tagTimeOffset - prevPoint.tagTimeOffset);
                progress = (prevPoint.strokeAdvance +
                    segmentProgress *
                    (point.strokeAdvance - prevPoint.strokeAdvance)) /
                strokesInfo[curStroke].strokeLength;
            } else {
                progress = 0;
            }

            animationState.curPoint = curPoint;
            animationState.curStroke = curStroke;

            renderer.update(curStroke, curPoint, progress);
        }
    }

    /**
     * Animation function after the graffiti has been drawn but rendering may
     * not be complete. The rendering will stop when the rendering is complete.
     */
    function fadeRendering () {
        if (renderer.fadeRendering() === false) {
            clearInterval(animationState.intervalId);
            animationState.intervalId = -1;
        }
    }

    /**
     * Called to initiate a recording a new graffiti drawing.
     */
    function record () {
        // If there is a pending animation, stop it now.
        if (animationState.intervalId !== -1) {
            clearInterval(animationState.intervalId);
            animationState.intervalId = -1;
        }

        // Show the play button, hid the record button
        Y.one("#play").setStyle('display', 'inline');
        Y.one('#record').setStyle('display', 'none');

        // Make sure the canvas is displayed
        canvas.setStyle('display', 'block');
        
        // Remove any existing graffiti
        g.removeAllChildren();

        // Now, clear the canvas for recording.
        ctx.clearRect(0, 0, canvas.get('width'), canvas.get('height'));
        ctx.globalAlpha = 1;
        ctx.fillStyle = RECORD_COLOR;

        // Instantiate a new GraffitiRecorder to record a new graffiti.
        recorder = new sw.GraffitiRecorder(Y.one('#graffitiViewer'));

        recorder.on(recorder.events.newPoint, function (event) {
            ctx.fillRect(event.x, event.y,
                         RECORD_MARKER_SIZE,
                         RECORD_MARKER_SIZE);
        });
    }
    
    /**
     * Called when the graffiti has been retrieved and pre-processed.
     *
     * @param gmlResult the GML data that was loaded and processed by the
     *        GraffitiLib
     */
    function onGMLLoaded (gmlResult) {
        var matrix = computeGraffitiTransform(gmlResult);

        // Reset state
        g.removeAllChildren();
        if (animationState.intervalId === -1) {
            clearInterval(animationState.intervalId);
        }

        ctx.clearRect(0, 0, canvas.get('width'), canvas.get('height'));
        ctx.fillStyle = PARTICLE_COLOR;
        ctx.lineSize = 0.5;
        ctx.strokeStyle = PARTICLE_COLOR;
        
        animationState = {
            curStroke: 0,
            curPoint: 0,
            strokesInfo: gmlResult.gmlMetrics.strokesInfo,
            startTime: (new Date()).getTime(),
            intervalId: -1
        };

        graffitiState = {
            matrix: matrix,
            origin: matrix.transformPoint({
                x: 0,
                y: 0
            }),
            gmlResult: gmlResult,
            paths: createGraffitiPaths(gmlResult)
        };
        
        // Check whether or not we should be showing the canvas
        canvas.setStyle('display',
                    Y.one('#show-particles').get('checked') === true ? 'block': 'none');

        renderer.initialize(graffitiState);
        animationState.intervalId = setInterval(animateStroke, animationRate);
    }

    function requestGML(graffitiId) {
        var gmlLib = new sw.graffiti.GraffitiLib();

        gmlLib.on(gmlLib.events.gmlLoaded, onGMLLoaded);
        gmlLib.request(graffitiId);
    }

    function requestGMLLocal(graffitiURL) {
        var gmlLib = new sw.graffiti.GraffitiLib();

        gmlLib.on(gmlLib.events.gmlLoaded, onGMLLoaded);
        gmlLib.requestLocal(graffitiURL);
    }

    function showStrokeSkeleton () {
        var strokeSkeleton = Y.one("#stroke-skeleton");
        if (strokeSkeleton !== undefined && strokeSkeleton !== null) {
            if (strokeSkeleton.getStyle("display") === "none") {
                strokeSkeleton.setStyle("display", "inline");
            } else {
                strokeSkeleton.setStyle("display", "none");
            }
        }
    }

    // Initialize form event listeners

    /**
     * When we click on a predefined graffiti id, we automatically fetch it.
     */
    Y.one("#predefined").on('click', function (event) {
        requestGML(event.target.get('id'));
    });
    Y.one("#predefinedLocal").on('click', function (event) {
        requestGMLLocal(event.target.get('id'));
    });
    Y.one("#random").on('click', function (event) {
        requestGML(Math.round(Math.random() * 14000));
    });
    Y.one("#record").on('click', function (event) {
        record();
    });

    function playLastRecorded () {
        if (recorder === null) {
            alert("There is no recorded graffiti.\n" +
                  "Click on the 'record' button \n" +
                  "and then click and drag you mouse \n" +
                  "on the canvs to create a graffiti");
        } else {
            Y.one("#play").setStyle("display", "none");
            Y.one("#record").setStyle("display", "inline");
            var gmlLib = new sw.graffiti.GraffitiLib();

            gmlLib.on(gmlLib.events.gmlLoaded, onGMLLoaded);
            gmlLib.onGraffitiLoad(recorder.getGMLRecord(), "record");
        }
    }
    Y.one("#play").on('click', playLastRecorded);
    Y.one("#last-recorded").on('click', playLastRecorded);
    Y.one('#to-svg').on('click', exportToSVG);
    Y.one('#show-gml').on('click', showGML);
    Y.one('#show-brick-overlay').on('click', showOverlay);
    Y.one('#close-capture').on('click', function () {
        var svgCapture = Y.one('#svg-capture'),
            captureBounds = svgCapture._node.getBoundingClientRect(),
            hideSVGCapture = new sw.animation.Animate({
                node: svgCapture,
                from: {
                    top: "50px"
                },
                to: {
                    top: (captureBounds.top - captureBounds.bottom) + "px"
                },
                duration: 0.5,
                easing: Y.Easing.backIn
            });
        hideSVGCapture.onEnd(function () {
            svgCapture.setStyle('display', 'none');
        });
        hideSVGCapture.run();
    });
    Y.one('#close-gml-capture').on('click', function () {
        var gmlCapture = Y.one('#gml-capture'),
            captureBounds = gmlCapture._node.getBoundingClientRect(),
            hideGMLCapture = new sw.animation.Animate({
                node: gmlCapture,
                from: {
                    top: "50px"
                },
                to: {
                    top: (captureBounds.top - captureBounds.bottom) + "px"
                },
                duration: 0.5,
                easing: Y.Easing.backIn
            });
        hideGMLCapture.onEnd(function () {
            gmlCapture.setStyle('display', 'none');
        });
        hideGMLCapture.run();
    });
    Y.one('#close-overlay').on('click', function () {
        var brickOverlay = Y.one('#brick-overlay'),
            overlayBounds = brickOverlay._node.getBoundingClientRect(),
            hideOverlay = new sw.animation.Animate({
                node: brickOverlay,
                from: {
                    top: "50px"
                },
                to: {
                    top: (overlayBounds.top - overlayBounds.bottom) + "px"
                },
                duration: 0.5,
                easing: Y.Easing.backIn
            });
        hideOverlay.onEnd(function () {
            brickOverlay.setStyle('display', 'none');
        });
        hideOverlay.run();
    });
    Y.one("#show-particles").on('click', function (evt) {
        var displayValue = 'none';
        if (evt.target.get('checked') === true) {
            displayValue = "block";
        }
        Y.one("#graffitis-canvas").setStyle('display', displayValue);
    });

    // Initialize default UI state.
    Y.one('#show-particles')._node.checked = true;
}


/**
 * Main entry point. Refer to the YUI documentation to understand how the
 * YUI framework loads scripts and handles dependencies.
 *
 * The following declares the modules required by the code (such as
 * 'anim-svg') and their dependencies.
 *
 * The onScriptsReady function is called when the DOM tree is ready.
 */
YUI({
    groups: {
            
        svg: {
            combine: false,
            base: "../res/3.0/scripts/yui/",
            modules:  {
                "node-svg": {
                    path: 'svg/node-svg-debug.js',
                    requires: ['node']
                },
                "dom-style-svg": {
                    path: 'svg/dom-style-svg-debug.js',
                    requires: ['node', 'dom-svg']
                },
                "dom-svg": {
                    path: 'svg/dom-svg-debug.js',
                    requires: ['dom']
                },
                "anim-timing": {
                    path: 'anim-timing/anim-timing-debug.js',
                    requires: ['anim', 'node-svg']
                },
                "anim-svg": {
                    path: 'svg/anim-svg-debug.js',
                    requires: ['anim', 'dom-style-svg']
                }
            }
        },
        demo: {
            base: "./",
            modules: {
                'graffiti-lib': {
                    path: 'graffiti-lib.js',
                    requires: ['node']
                },
                'graffitis-record': {
                    path: 'graffitis-record.js',
                    requires: ['node', 'event-custom']
                },
                'smooth': {
                    path: 'smooth.js'
                }
            }
        }
    },
    filter: 'debug'
}).use('dom-svg', 'node-svg', 'dom-style-svg', 'anim-svg', 'anim-timing',
       'graffiti-lib', 'smooth', 'graffitis-record', function(YUI) {
    YUI.on('contentready', function () {
        onReady(YUI);
    }, 'body');
});